<?php
/**
 * CakePHP(tm) : Rapid Development Framework (https://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (https://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright Copyright (c) Cake Software Foundation, Inc. (https://cakefoundation.org)
 * @link      https://cakephp.org CakePHP(tm) Project
 * @since     0.2.9
 * @license   https://opensource.org/licenses/mit-license.php MIT License
 */
namespace App\Controller;

use Cake\Core\Configure;
use Cake\Http\Exception\ForbiddenException;
use Cake\Http\Exception\NotFoundException;
use Cake\View\Exception\MissingTemplateException;
use Cake\Mailer\Email;
use Cake\Mailer\TransportFactory;
#use DataTables\Controller\DataTablesAjaxRequestTrait;

require_once '../vendor/paypal/vendor/autoload.php';

/**
 * Static content controller
 *
 * This controller will render views from Template/Pages/
 *
 * @link https://book.cakephp.org/3/en/controllers/pages-controller.html
 */
class PagesController extends AppController
{

    /**
     * Displays a view
     *
     * @param array ...$path Path segments.
     * @return \Cake\Http\Response|null
     * @throws \Cake\Http\Exception\ForbiddenException When a directory traversal attempt.
     * @throws \Cake\Http\Exception\NotFoundException When the view file could not
     *   be found
     * @throws \Cake\View\Exception\MissingTemplateException In debug mode.
     */

    public $paginate = [
        'limit' => 15
    ];

    public function initialize()
    {
        parent::initialize();

        $this->loadComponent('Paginator');
        
        $this->Auth->allow(['paypalReceiver', 'paypalReceiverCancel', 'getPayment']);

    }

    public function display(...$path)
    {
        if (!$path) {
            return $this->redirect('/');
        }
        if (in_array('..', $path, true) || in_array('.', $path, true)) {
            throw new ForbiddenException();
        }
        $page = $subpage = null;

        if (!empty($path[0])) {
            $page = $path[0];
        }
        if (!empty($path[1])) {
            $subpage = $path[1];
        }
        $this->set(compact('page', 'subpage'));

        try {
            $this->render(implode('/', $path));
        } catch (MissingTemplateException $exception) {
            if (Configure::read('debug')) {
                throw $exception;
            }
            throw new NotFoundException();
        }
    }

    public function index(){

        $list = $this->payments('', false);

        $paidPendients = $this->pendients('', false);

        $paidComplete = $this->complete('', false);
        
        $this->loadModel('pagos');       
        $paid = $this->pagos->find('all')
                            ->select(['Monto' => 'concat("USD $",format(sum(price),0))'])
                            ->where(['status' => 'paid'])->first();
        $unpaid = $this->pagos->find('all')
                            ->select(['Monto' => 'concat("USD $",format(sum(price),0))'])
                            ->where(['status' => 'unpaid'])->first();
        $total = $this->pagos->find('all')
                            ->select(['Monto' => 'concat("USD $",format(sum(price),0))'])
                            ->where(['status !=' => ''])->first();

        $this->set(compact('paid', 'unpaid', 'total', 'list', 'paidPendients', 'paidComplete'));
    }

    public function payments($term = '', $echo = true){
        $this->loadModel('payments');

        if(empty($term)){
            $where = ['user_id' => $this->user_id];
        }else{
            $where = ['user_id' => $this->user_id, 'OR' => [['name LIKE' => '%'.$term.'%'], ['rut LIKE' => '%'.$term.'%'], ['email LIKE' => '%'.$term.'%'], ['date LIKE' => '%'.$term.'%']]]; 
        }

        $list = $this->payments->find('all')->where($where);
        $list = $this->paginate($list, [
            'orden' => [
                'date' => 'desc'
            ]
        ]);

        if($echo){
            return $this->response->withType('application/json')->withStringBody(json_encode($list));
        }

        return $list;
    }

    public function pendients($term = '', $echo = true){
        $this->loadModel('pendients');

        if(empty($term)){
            $where = ['user_id' => $this->user_id];
        }else{
            $where = ['user_id' => $this->user_id, 'OR' => [['name LIKE' => '%'.$term.'%'], ['rut LIKE' => '%'.$term.'%'], ['email LIKE' => '%'.$term.'%'], ['create_date LIKE' => '%'.$term.'%']]]; 
        }

        $paidPendients = $this->pendients->find('all')->where($where);
        $paidPendients = $this->paginate($paidPendients);

        if($echo){
            return $this->response->withType('application/json')->withStringBody(json_encode($paidPendients));
        }

        return $paidPendients;
    }

    public function complete($term = '', $echo = true){
        $this->loadModel('paid');

        if(empty($term)){
            $where = ['user_id' => $this->user_id];
        }else{
            $where = ['user_id' => $this->user_id, 'OR' => [['name LIKE' => '%'.$term.'%'], ['rut LIKE' => '%'.$term.'%'], ['email LIKE' => '%'.$term.'%'], ['paydate LIKE' => '%'.$term.'%']]]; 
        }

        $paidComplete = $this->paid->find('all')->where($where);
        $paidComplete = $this->paginate($paidComplete);

        if($echo){
            return $this->response->withType('application/json')->withStringBody(json_encode($paidComplete));
        }

        return $paidComplete;
    }
    
    public function addPayment(){
        
        if($this->request->is('post')){
            $this->loadModel('transactions');
            $transaction = $this->transactions->newEntity();
            $data = $this->request->getData();
            
            $transaction->name = $data['name'];
            $transaction->rut = $data['rut'];
            $transaction->email = $data['email'];
            $transaction->price = $data['price'];
            $transaction->user_id = $this->user_id;
            $transaction->created = date('Y-m-d H:i:s');
            
            if($this->transactions->save($transaction)){
                $trans = $this->transactions->find('all')->select('trans_id')->where(['user_id' => $this->user_id])->enableHydration(false)->max('trans_id');
                
                $this->generatePayment($trans['trans_id'], false);                

                $this->Flash->success('Payment registered!');
                $this->redirect(['controller' => 'Pages', 'action' => 'index']);
            }else{
                $this->Flash->error('Payment unregistered!');
            }
        }
        
    }

    public function generatePayment( $id = null, $redirect = true){
        $this->autoRender = false;

        if(!$id){
            $this->Flash->error('Error request!');
            $this->redirect(['action' => 'index']);
        }

        $this->loadModel('settings');

        $paypal = $this->settings->find('all')->select(['paypal_client', 'paypal_client_secret'])->where(['user_id' => $this->user_id])->first();
        
        if(!$paypal){
            $this->Flash->error('Please, review settings and try again! Mail not sent, you must send it manual!');
            return $this->redirect(['action' => 'index']);
        }

        $apiContext = new \PayPal\Rest\ApiContext(
            new \PayPal\Auth\OAuthTokenCredential(
                $paypal->paypal_client,     // ClientID
                $paypal->paypal_client_secret     // ClientSecret
            )
        );

        $this->loadModel('transactions');

        $data = $this->transactions->find('all')->where(['trans_id' => $id])->first();

        $price = $data['price'];
        
        $payer = new \PayPal\Api\Payer();
        $payer->setPaymentMethod('paypal');

        $amount = new \PayPal\Api\Amount();
        $amount->setTotal($price);
        $amount->setCurrency('USD');

        $transaction = new \PayPal\Api\Transaction();
        $transaction->setAmount($amount);

        $redirectUrls = new \PayPal\Api\RedirectUrls();
        $redirectUrls->setReturnUrl('https://dp.imexdbusiness.com/paypal-receiver/'.$id/*HACER PAGINA DE RETORNO*/)
            ->setCancelUrl('https://dp.imexdbusiness.com/paypal-receiver-cancel/'.$id/*PAGINA DE ITFINDEN*/);

        $payment = new \PayPal\Api\Payment();
        $payment->setIntent('sale')
            ->setPayer($payer)
            ->setTransactions([$transaction])
            ->setRedirectUrls($redirectUrls);
        // After Step 3
        try {
            $payment->create($apiContext);

            $return = [ 'url' => $payment->getApprovalLink(), 'payment_id' => $payment->getId(), 'transaction' => $id ];
            
            $code = hash('crc32b', $return['payment_id']);
            $this->transactions->updateAll(['code' => $code], ['trans_id' => $id]);

            $this->loadModel('paypal_payments');
            
            $existsPayment = $this->paypal_payments->find('all')->where(['trans_id' => $id])->count();
            if(!$existsPayment){
                $paypal = $this->paypal_payments->newEntity(); 
                
                $paypal->payment_id = $return['payment_id'];
                $paypal->trans_id = $id;
                $paypal->url = $return['url'];
                $paypal->user_id = $this->user_id;
                $paypal->create_date = date('Y-m-d H:i:s');
    
                if($this->paypal_payments->save($paypal)){
                    $this->Flash->success('Link generated OK!');
                }else{
                    $this->Flash->success('Link not generated!');
                }
            }else{
               $this->paypal_payments->updateAll(['payment_id' => $return['payment_id'], 'url' => $return['url'], 'create_date' => date('Y-m-d H:i:s')], ['trans_id' => $id]);
               $this->Flash->success('Link generated OK!');
            }
            
            $this->sendPayment($code, false);

            if($redirect)
            return $this->redirect(['action' => 'index']);
        }
        catch (\PayPal\Exception\PayPalConnectionException $ex) {
            //This will print the detailed information on the exception.
            //REALLY HELPFUL FOR DEBUGGING
            return $ex->getData();
        }
    }

    public function regeneratePayment($id = null){
        
        $this->generatePayment($id);

        $this->Flash->success('Regenerated payment link!');
        return $this->redirect(['action' => 'index']);
    }
    
    public function unlinkPayment($id = null){
        
        $this->loadModel('paypal_payments');
        $this->paypal_payments->updateAll(['trans_id' => NULL], ['trans_id' => $id]);
        
        $this->loadModel('transactions');
        $this->transactions->updateAll(['code' => NULL], ['trans_id' => $id]);

        $this->Flash->success('Unlinked payment!');
        return $this->redirect(['action' => 'index']);
    }

    public function sendPayment($code = null, $redirect = true){
        $this->loadModel('payment_info');
        $this->loadModel('settings');

        $this->user_id = $this->Auth->user('id');

        $info = $this->payment_info->find('all')->where(['code' => $code])->first();
        $setting = $this->settings->find('all')->where(['user_id' => $this->user_id])->first();
        
        TransportFactory::setConfig('sendmail', [
            'host' => $setting->host,
            'port' => $setting->port,
            'username' => $setting->email,
            'password' => $setting->password,
            'className' => 'Smtp'
        ]);

        /*enviando el correo*/
        $email = new Email(); //instancia de correo
        $email
            ->setTransport('sendmail') //nombre del configTrasnport que acabamos de configurar
            ->setEmailFormat('html') //formato de correo
            ->setTo($info->email) //correo para
            ->setFrom($setting->email) //correo de
            ->setSubject('Notification ImexDBusiness') //asunto
            ->setViewVars([ //enviar variables a la plantilla
                'client' => $info->name,
                'invoice_num' => $info->code,
                'invoice_date_created' => date('Y-m-d H:i:s'),
                'code' => $info->code,
                'url' => $info->url
            ])->viewBuilder()->setTemplate('payment'); //plantilla a utilizar;

        if($email->send()){
            $this->Flash->success('Mail sent');
        }else{
            $this->Flash->error('Mail not sent');
        }

        if($redirect)
        return $this->redirect(['action' => 'index']);
    }

    public function getPayment( $code = null){

        $this->viewBuilder()->enableAutoLayout(false);
        if($this->request->is('post')){
            $this->loadModel('payment_info');

            $info = $this->payment_info->find('all')->select(['url'])->where(['code' => $code])->first();

            if($info){
                $data = json_encode($info);
            }else{
                $data = json_decode(['url' => 'empty']);
            }
            return $this->response->withType('application/json')->withStringBody($data);
        }

    }

    public function paypalReceiver( $id = null ){
        
        $paymentId = $this->request->getQuery('paymentId');
        
        $this->loadModel('paypal_payments');
        $data = $this->paypal_payments->find('all')->where(['payment_id' => $paymentId])->first();
        
        if (!$data){
            $this->Flash->error('Error request to save data of payment, contact us (Payment ID '.$paymentId.')');
            return $this->redirect(['action' => 'index']);
        }            

        $this->paypal_payments->updateAll(['status' => 'paid'], ['payment_id' => $paymentId]);

        $trans = $data['trans_id'];

        $this->loadModel('transactions');
        $dataTrans = $this->transactions->find('all')->where(['trans_id' => $trans])->first();

        $attr = json_decode($dataTrans['attr'],true);
        $attr['paydate'] = date('Y-m-d H:i:s');
        $attr = json_encode($attr); 

        $this->transactions->updateAll(['attr' => $attr], ['trans_id' => $trans]);

        $this->Flash->success('Notification received!');
        return $this->redirect(['action' => 'index']);
    }

    public function paypalReceiverCancel( $id = null){
        $this->Flash->error('Payment cancelled');
        return $this->redirect(['action' => 'index']);
    }

    public function deletePayment($id = null)
    {
        $this->request->allowMethod(['post', 'delete']);
        $this->loadModel('transactions');
        $payment = $this->transactions->find('all')->where(['trans_id'=> $id])->first();
        if ($this->transactions->delete($payment)) {
            $this->Flash->success(__('The payment has been deleted.'));
        } else {
            $this->Flash->error(__('The payment could not be deleted. Please, try again.'));
        }

        return $this->redirect(['action' => 'index']);
    }

    public function editPayment($id = null)
    {
        $this->loadModel('transactions');
        $payment = $this->transactions->find('all')->where(['trans_id'=> $id])->first();
        if ($this->request->is(['patch', 'post', 'put'])) {
            $payment = $this->transactions->patchEntity($payment, $this->request->getData());
            if ($this->transactions->save($payment)) {
                $this->Flash->success(__('The payment has been saved.'));

                return $this->redirect(['action' => 'index']);
            }
            $this->Flash->error(__('The payment could not be saved. Please, try again.'));
        }
        
        $this->set(compact('payment'));
    }

    public function settPayment(){
        $this->loadModel('settings');
        $this->user_id = $this->Auth->user('id');

        $settings = $this->settings->find('all')->where(['user_id' => $this->user_id])->first();
        
        if(!$settings){
            $this->Flash->error('Please, review settings and try again!');
            $this->redirect(['action' => 'index']);
        }

        if($this->request->is(['post', 'put'])){
            $data = $this->request->getData();
            if(!$settings){
                $settings = $this->settings->newEntity();
            }

            $settings->user_id = $this->user_id;
            $settings->port = $data['port'];
            $settings->host = $data['host'];
            $settings->email = $data['email'];
            $settings->password = $data['password'];
            $settings->paypal_client = $data['paypal_client'];
            $settings->paypal_client_secret = $data['paypal_client_secret'];

            if($this->settings->save($settings)){
                $this->Flash->success('Data saved!');
            }else{
                $this->Flash->success('Ups, try again!');
            }

            return $this->redirect(['action' => 'index']);
        }        

        $this->set(compact('settings'));
    }
}
